import json
from unittest import TestCase

import numpy as np

from scattertext import LogOddsRatioUninformativeDirichletPrior
from scattertext import ScatterChart
from scattertext.ScatterChart import CoordinatesNotRightException
from scattertext.test.test_termDocMatrixFactory \
	import build_hamlet_jz_term_doc_mat, build_hamlet_jz_corpus_with_meta


class TestScatterChart(TestCase):
	def test_to_json(self):
		tdm = build_hamlet_jz_term_doc_mat()
		# with self.assertRaises(NoWordMeetsTermFrequencyRequirementsError):
		#	ScatterChart(term_doc_matrix=tdm).to_dict('hamlet')
		j = (ScatterChart(term_doc_matrix=tdm,
		                  minimum_term_frequency=0)
		     .to_dict('hamlet'))
		self.assertEqual(set(j.keys()), set(['info', 'data']))
		self.assertEqual(set(j['info'].keys()),
		                 set(['not_category_name', 'category_name',
		                      'category_terms', 'not_category_terms', 'category_internal_name']))
		expected = {"x": 0.0,
		            "y": 0.42,
		            "term": "art",
		            "cat25k": 758,
		            "ncat25k": 0,
		            's': 0.5,
		            'os': 3,
		            'bg': 3}
		datum = self._get_data_example(j)
		for var in ['cat25k', 'ncat25k']:
			np.testing.assert_almost_equal(expected[var], datum[var], decimal=1)
		self.assertEqual(set(expected.keys()), set(datum.keys()))
		self.assertEqual(expected['term'], datum['term'])

	def _get_data_example(self, j):
		return [t for t in j['data'] if t['term'] == 'art'][0]

	def test_p_vals(self):
		tdm = build_hamlet_jz_term_doc_mat()
		j = (ScatterChart(term_doc_matrix=tdm,
		                  minimum_term_frequency=0,
		                  term_significance=LogOddsRatioUninformativeDirichletPrior())
		     .to_dict('hamlet'))
		datum = self._get_data_example(j)
		self.assertIn('p', datum.keys())

	def test_inject_coordinates(self):
		tdm = build_hamlet_jz_term_doc_mat()
		freq_df = tdm.get_term_freq_df()
		scatter_chart = ScatterChart(term_doc_matrix=tdm,
		                             minimum_term_frequency=0)
		with self.assertRaises(CoordinatesNotRightException):
			scatter_chart.inject_coordinates([], [])
		with self.assertRaises(CoordinatesNotRightException):
			scatter_chart.inject_coordinates(freq_df[freq_df.columns[0]], [])
		with self.assertRaises(CoordinatesNotRightException):
			scatter_chart.inject_coordinates([], freq_df[freq_df.columns[0]])
		x = freq_df[freq_df.columns[1]].astype(np.float)
		y = freq_df[freq_df.columns[0]].astype(np.float)
		with self.assertRaises(CoordinatesNotRightException):
			scatter_chart.inject_coordinates(x, y)
		with self.assertRaises(CoordinatesNotRightException):
			scatter_chart.inject_coordinates(x, y/y.max())
		with self.assertRaises(CoordinatesNotRightException):
			scatter_chart.inject_coordinates(x/x.max(), y)
		with self.assertRaises(CoordinatesNotRightException):
			scatter_chart.inject_coordinates(-x / x.max(), -y / y.max())
		with self.assertRaises(CoordinatesNotRightException):
			scatter_chart.inject_coordinates(-x / x.max(), y / y.max())
		with self.assertRaises(CoordinatesNotRightException):
			scatter_chart.inject_coordinates(x / x.max(), -y / y.max())
		scatter_chart.inject_coordinates(x / x.max(), y / y.max())

	def test_to_json_use_non_text_features(self):
		tdm = build_hamlet_jz_corpus_with_meta()
		# with self.assertRaises(NoWordMeetsTermFrequencyRequirementsError):
		#	ScatterChart(term_doc_matrix=tdm).to_dict('hamlet')
		j = (ScatterChart(term_doc_matrix=tdm,
		                  minimum_term_frequency=0,
		                  use_non_text_features=True)
		     .to_dict('hamlet'))
		self.assertEqual(set(j.keys()), set(['info', 'data']))
		self.assertEqual(set(j['info'].keys()),
		                 set(['not_category_name', 'category_name',
		                      'category_terms', 'not_category_terms',
		                      'category_internal_name']))
		self.assertEqual({t['term'] for t in j['data']}, {'cat1'}
		                 #{'cat4', 'cat9', 'cat5', 'cat0', 'cat3', 'cat2', 'cat1'}
		                 )
		json.dumps(j)

	def test_max_terms(self):
		tdm = build_hamlet_jz_term_doc_mat()
		# with self.assertRaises(NoWordMeetsTermFrequencyRequirementsError):
		#	ScatterChart(term_doc_matrix=tdm).to_dict('hamlet')
		j = (ScatterChart(term_doc_matrix=tdm,
		                  minimum_term_frequency=0,
		                  max_terms=2)
		     .to_dict('hamlet'))
		self.assertEqual(2, len(j['data']))

		j = (ScatterChart(term_doc_matrix=tdm,
		                  minimum_term_frequency=0,
		                  max_terms=10)
		     .to_dict('hamlet'))
		self.assertEqual(10, len(j['data']))

		j = (ScatterChart(term_doc_matrix=tdm,
		                  minimum_term_frequency=0,
		                  max_terms=10000)
		     .to_dict('hamlet'))
		self.assertEqual(51, len(j['data']))

		j = (ScatterChart(term_doc_matrix=tdm,
		                  minimum_term_frequency=0,
		                  max_terms=None)
		     .to_dict('hamlet'))
		self.assertEqual(51, len(j['data']))



import spacy
from gensim.models import word2vec

from scattertext import SampleCorpora, word_similarity_explorer_gensim, Word2VecFromParsedCorpus
from scattertext.CorpusFromParsedDocuments import CorpusFromParsedDocuments


def main():
	nlp = spacy.en.English()
	convention_df = SampleCorpora.ConventionData2012.get_data()
	convention_df['parsed'] = convention_df.text.apply(nlp)
	corpus = CorpusFromParsedDocuments(convention_df,
	                                   category_col='party',
	                                   parsed_col='parsed').build()
	model = word2vec.Word2Vec(size=300,
	                          alpha=0.025,
	                          window=5,
	                          min_count=5,
	                          max_vocab_size=None,
	                          sample=0,
	                          seed=1,
	                          workers=1,
	                          min_alpha=0.0001,
	                          sg=1,
	                          hs=1,
	                          negative=0,
	                          cbow_mean=0,
	                          iter=1,
	                          null_word=0,
	                          trim_rule=None,
	                          sorted_vocab=1)
	html = word_similarity_explorer_gensim(corpus,
	                                       category='democrat',
	                                       category_name='Democratic',
	                                       not_category_name='Republican',
	                                       target_term='jobs',
	                                       minimum_term_frequency=5,
	                                       pmi_filter_thresold=4,
	                                       width_in_pixels=1000,
	                                       metadata=convention_df['speaker'],
	                                       word2vec=Word2VecFromParsedCorpus(corpus, model).train(),
	                                       max_p_val=0.05,
	                                       save_svg_button=True)
	open('./demo_gensim_similarity.html', 'wb').write(html.encode('utf-8'))
	print('Open ./demo_gensim_similarity.html in Chrome or Firefox.')


if __name__ == '__main__':
	main()




from scattertext import SampleCorpora
from scattertext import produce_scattertext_explorer
from scattertext.CorpusFromPandas import CorpusFromPandas
from scattertext.WhitespaceNLP import whitespace_nlp

nlp = whitespace_nlp

convention_df = SampleCorpora.ConventionData2012.get_data()
corpus = CorpusFromPandas(convention_df,
                          category_col='party',
                          text_col='text',
                          nlp=nlp).build()

html = produce_scattertext_explorer(corpus,
                                    category='democrat',
                                    category_name='Democratic',
                                    not_category_name='Republican',
                                    minimum_term_frequency=5,
                                    width_in_pixels=1000,
                                    metadata=convention_df['speaker'])
open('./demo_without_spacy.html', 'wb').write(html.encode('utf-8'))
print('Open ./demo_without_spacy.html in Chrome or Firefox.')




# Constants for scaled f-score
DEFAULT_BETA = 0.5
DEFAULT_SCALER_ALGO = 'normcdf'

# Constants for background scaled f-score
DEFAULT_BACKGROUND_BETA = 1
DEFAULT_BACKGROUND_SCALER_ALGO = 'none'

# For sample corpus
POLITICAL_DATA_URL = 'https://gitcdn.xyz/repo/JasonKessler/scattertext/master/scattertext/data/political_data.json'

# For sample corpus
DEFAULT_D3_URL \
	= 'http://cdnjs.cloudflare.com/ajax/libs/d3/4.6.0/d3.min.js'
DEFAULT_D3_SCALE_CHROMATIC \
	= 'http://d3js.org/d3-scale-chromatic.v1.min.js'



from unittest import TestCase

from scattertext.termsignificance import LogOddsRatioUninformativeDirichletPrior
from scattertext.test.test_termDocMatrixFactory import build_hamlet_jz_term_doc_mat


class TestLogOddsRatioUninformativeDirichletPrior(TestCase):
	def test_get_p_vals(self):
		tdm = build_hamlet_jz_term_doc_mat()
		df = tdm.get_term_freq_df()
		X = df[['hamlet freq', 'jay-z/r. kelly freq']].values
		pvals = LogOddsRatioUninformativeDirichletPrior().get_p_vals(X)
		self.assertGreaterEqual(min(pvals), 0)
		self.assertLessEqual(min(pvals), 1)




from __future__ import print_function

import warnings

import numpy as np

import scattertext.viz
from scattertext import SampleCorpora
from scattertext import Scalers, ScatterChart
from scattertext import termranking
from scattertext.AsianNLP import chinese_nlp, japanese_nlp
from scattertext.CSRMatrixTools import CSRMatrixFactory
from scattertext.CorpusFromPandas import CorpusFromPandas
from scattertext.CorpusFromParsedDocuments import CorpusFromParsedDocuments
from scattertext.IndexStore import IndexStore
from scattertext.ParsedCorpus import ParsedCorpus
from scattertext.Scalers import percentile_alphabetical
from scattertext.ScatterChart import ScatterChart
from scattertext.ScatterChartExplorer import ScatterChartExplorer
from scattertext.TermDocMatrix import TermDocMatrix
from scattertext.TermDocMatrixFactory import TermDocMatrixFactory, FeatsFromDoc
from scattertext.TermDocMatrixFilter import TermDocMatrixFilter, filter_bigrams_by_pmis
from scattertext.TermDocMatrixFromPandas import TermDocMatrixFromPandas
from scattertext.WhitespaceNLP import whitespace_nlp, whitespace_nlp_with_sentences, tweet_tokenzier_factory
from scattertext.features.FeatsFromOnlyEmpath import FeatsFromOnlyEmpath
from scattertext.features.FeatsFromSpacyDoc import FeatsFromSpacyDoc
from scattertext.features.FeatsFromSpacyDocAndEmpath import FeatsFromSpacyDocAndEmpath
from scattertext.features.FeatsFromSpacyDocOnlyEmoji import FeatsFromSpacyDocOnlyEmoji
from scattertext.features.FeatsFromSpacyDocOnlyNounChunks import FeatsFromSpacyDocOnlyNounChunks
from scattertext.representations.Word2VecFromParsedCorpus import Word2VecFromParsedCorpus, \
	Word2VecFromParsedCorpusBigrams
from scattertext.termranking import OncePerDocFrequencyRanker
from scattertext.termscoring.ScaledFScore import InvalidScalerException
from scattertext.termsignificance.LogOddsRatioUninformativeDirichletPrior import LogOddsRatioUninformativeDirichletPrior
from scattertext.viz import VizDataAdapter, HTMLVisualizationAssembly
version = [0,0,2,9]

def produce_scattertext_html(term_doc_matrix,
                             category,
                             category_name,
                             not_category_name,
                             protocol='https',
                             minimum_term_frequency=3,
                             pmi_threshold_coefficient=6,
                             max_terms=None,
                             filter_unigrams=False,
                             height_in_pixels=None,
                             width_in_pixels=None,
                             term_ranker=termranking.AbsoluteFrequencyRanker):
	'''Returns html code of visualization.

	Parameters
	----------
	term_doc_matrix : TermDocMatrix
		Corpus to use
	category : str
		name of category column
	category_name: str
		name of category to mine for
	not_category_name: str
		name of everything that isn't in category
	protocol : str
		optional, used prototcol of , http or https
	minimum_term_frequency : int, optional
		Minimum number of times word needs to appear to make it into visualization.
	pmi_threshold_coefficient : int, optional
		Filter out bigrams with a PMI of < 2 * pmi_threshold_coefficient. Default is 6.
	max_terms : int, optional
		Maximum number of terms to include in visualization.
	filter_unigrams : bool
		default False, do we filter unigrams that only occur in one bigram
	width_in_pixels: int
		width of viz in pixels, if None, default to JS's choice
	height_in_pixels: int
		height of viz in pixels, if None, default to JS's choice
	term_ranker : TermRanker
			TermRanker class for determining term frequency ranks.

	Returns
	-------
		str, html of visualization
	'''
	scatter_chart_data = ScatterChart(term_doc_matrix=term_doc_matrix,
	                                  minimum_term_frequency=minimum_term_frequency,
	                                  pmi_threshold_coefficient=pmi_threshold_coefficient,
	                                  filter_unigrams=filter_unigrams,
	                                  max_terms=max_terms,
	                                  term_ranker=term_ranker) \
		.to_dict(category=category,
	           category_name=category_name,
	           not_category_name=not_category_name,
	           transform=percentile_alphabetical)
	html = HTMLVisualizationAssembly(VizDataAdapter(scatter_chart_data),
	                                 width_in_pixels,
	                                 height_in_pixels).to_html(protocol=protocol)
	return html


def produce_scattertext_explorer(corpus,
                                 category,
                                 category_name,
                                 not_category_name,
                                 protocol='https',
                                 pmi_threshold_coefficient=6,
                                 minimum_term_frequency=3,
                                 minimum_not_category_term_frequency=0,
                                 max_terms=None,
                                 filter_unigrams=False,
                                 height_in_pixels=None,
                                 width_in_pixels=None,
                                 max_snippets=None,
                                 max_docs_per_category=None,
                                 metadata=None,
                                 scores=None,
                                 x_coords=None,
                                 y_coords=None,
                                 singleScoreMode=False,
                                 sort_by_dist=True,
                                 reverse_sort_scores_for_not_category=True,
                                 use_full_doc=False,
                                 transform=percentile_alphabetical,
                                 jitter=0,
                                 grey_zero_scores=False,
                                 term_ranker=None,
                                 asian_mode=False,
                                 use_non_text_features=False,
                                 show_characteristic=True,
                                 word_vec_use_p_vals=False,
                                 max_p_val=0.05,
                                 p_value_colors=False,
                                 term_significance=None,
                                 save_svg_button=False,
                                 x_label=None,
                                 y_label=None,
                                 d3_url=None,
                                 d3_scale_chromatic_url=None,
                                 pmi_filter_thresold=None):
	'''Returns html code of visualization.

	Parameters
	----------
	corpus : Corpus
		Corpus to use.
	category : str
		Name of category column as it appears in original data frame.
	category_name : str
		Name of category to use.  E.g., "5-star reviews."
	not_category_name : str
		Name of everything that isn't in category.  E.g., "Below 5-star reviews".
	protocol : str, optional
		Protocol to use.  Either http or https.  Default is https.
	pmi_threshold_coefficient : int, optional
		Filter out bigrams with a PMI of < 2 * pmi_threshold_coefficient. Default is 6
	minimum_term_frequency : int, optional
		Minimum number of times word needs to appear to make it into visualization.
	minimum_not_category_term_frequency : int, optional
	  If an n-gram does not occur in the category, minimum times it
	   must been seen to be included. Default is 0.
	max_terms : int, optional
		Maximum number of terms to include in visualization.
	filter_unigrams : bool, optional
		Default False, do we filter out unigrams that only occur in one bigram
	width_in_pixels : int, optional
		Width of viz in pixels, if None, default to JS's choice
	height_in_pixels : int, optional
		Height of viz in pixels, if None, default to JS's choice
	max_snippets : int, optional
		Maximum number of snippets to show when term is clicked.  If None, all are shown.
	max_docs_per_category: int, optional
		Maximum number of documents to store per category.  If None, by default, all are stored.
	metadata : list, optional
		list of meta data strings that will be included for each document
	scores : np.array, optional
		Array of term scores or None.
	x_coords : np.array, optional
		Array of term x-axis positions or None.  Must be in [0,1].
		If present, y_coords must also be present.
	y_coords : np.array, optional
		Array of term y-axis positions or None.  Must be in [0,1].
		If present, x_coords must also be present.
	singleScoreMode : bool, optional
		Label terms based on score vs distance from corner.  Good for topic scores. Show only one color.
	sort_by_dist: bool, optional
		Label terms based distance from corner. True by default.  Negated by singleScoreMode.
	reverse_sort_scores_for_not_category: bool, optional
		If using a custom score, score the not-category class by
		lowest-score-as-most-predictive. Turn this off for word vectory
		or topic similarity. Default True.
	use_full_doc : bool, optional
		Use the full document in snippets.  False by default.
	transform : function, optional
		not recommended for editing.  change the way terms are ranked.  default is st.Scalers.percentile_ordinal
	jitter : float, optional
		percentage of axis to jitter each point.  default is 0.
	grey_zero_scores : bool, optional
		If True, color points with zero-scores a light shade of grey.  False by default.
	term_ranker : TermRanker, optional
		TermRanker class for determining term frequency ranks.
	asian_mode : bool, optional
		Use a special Javascript regular expression that's specific to chinese or japanese
	use_non_text_features : bool, optional
		Show non-bag-of-words features (e.g., Empath) instaed of text.  False by default.
	show_characteristic: bool, default True
		Show characteristic terms on the far left-hand side of the visualization
	word_vec_use_p_vals: bool, default False
		Sort by harmonic mean of score and distance.
	max_p_val : float, default 0.05
		If word_vec_use_p_vals, the minimum p val to use.
	p_value_colors : bool, default False
	  Color points differently if p val is above 1-max_p_val, below max_p_val, or
	   in between.
	term_significance : TermSignifiance instance or None
		Way of getting signfiance scores.  If None, p values will not be added.
	save_svg_button : bool, default False
		Add a save as SVG button to the page.
	x_label : str, default None
		Custom x-axis label
	y_label : str, default None
		Custom y-axis label
	d3_url, str, None by default.  The url (or path) of d3.
		URL of d3, to be inserted into <script src="..."/>.  Overrides `protocol`.
	  By default, this is `DEFAULT_D3_URL` declared in `HTMLVisualizationAssembly`.
	d3_scale_chromatic_url, str, None by default.  Overrides `protocol`.
	  URL of d3 scale chromatic, to be inserted into <script src="..."/>
	  By default, this is `DEFAULT_D3_SCALE_CHROMATIC` declared in `HTMLVisualizationAssembly`.
	pmi_filter_thresold : (DEPRECATED) int, None by default
	  DEPRECATED.  Use pmi_threshold_coefficient instead.
	Returns
	-------
		str, html of visualization

	'''
	color = None
	if singleScoreMode or word_vec_use_p_vals:
		color = 'd3.interpolatePurples'
	if singleScoreMode or not sort_by_dist:
		sort_by_dist = False
	else:
		sort_by_dist = True
	if term_ranker is None:
		term_ranker = termranking.AbsoluteFrequencyRanker

	if pmi_filter_thresold is not None:
		pmi_threshold_coefficient = pmi_filter_thresold
		warnings.warn(
			"The argument name 'pmi_filter_thresold' has been deprecated. Use 'pmi_threshold_coefficient' in its place",
			DeprecationWarning)

	scatter_chart_explorer = ScatterChartExplorer(corpus,
	                                              minimum_term_frequency=minimum_term_frequency,
	                                              minimum_not_category_term_frequency=minimum_not_category_term_frequency,
	                                              pmi_threshold_coefficient=pmi_threshold_coefficient,
	                                              filter_unigrams=filter_unigrams,
	                                              jitter=jitter,
	                                              max_terms=max_terms,
	                                              term_ranker=term_ranker,
	                                              use_non_text_features=use_non_text_features,
	                                              term_significance=term_significance)
	if ((x_coords is None and y_coords is not None)
	    or (y_coords is None and x_coords is not None)):
		raise Exception("Both x_coords and y_coords need to be passed or both left blank")
	if x_coords is not None:
		scatter_chart_explorer.inject_coordinates(x_coords, y_coords)
	scatter_chart_data = scatter_chart_explorer.to_dict(category=category,
	                                                    category_name=category_name,
	                                                    not_category_name=not_category_name,
	                                                    transform=transform,
	                                                    scores=scores,
	                                                    max_docs_per_category=max_docs_per_category,
	                                                    metadata=metadata)
	return HTMLVisualizationAssembly(VizDataAdapter(scatter_chart_data),
	                                 width_in_pixels=width_in_pixels,
	                                 height_in_pixels=height_in_pixels,
	                                 max_snippets=max_snippets,
	                                 color=color,
	                                 grey_zero_scores=grey_zero_scores,
	                                 sort_by_dist=sort_by_dist,
	                                 reverse_sort_scores_for_not_category=reverse_sort_scores_for_not_category,
	                                 use_full_doc=use_full_doc,
	                                 asian_mode=asian_mode,
	                                 use_non_text_features=use_non_text_features,
	                                 show_characteristic=show_characteristic,
	                                 word_vec_use_p_vals=word_vec_use_p_vals,
	                                 max_p_val=max_p_val,
	                                 save_svg_button=save_svg_button,
	                                 p_value_colors=p_value_colors,
	                                 x_label=x_label,
	                                 y_label=y_label) \
		.to_html(protocol=protocol,
	           d3_url=d3_url,
	           d3_scale_chromatic_url=d3_scale_chromatic_url)


def word_similarity_explorer_gensim(corpus,
                                    category,
                                    category_name,
                                    not_category_name,
                                    target_term,
                                    word2vec=None,
                                    alpha=0.01,
                                    max_p_val=0.05,
                                    **kwargs):
	'''
		Parameters
		----------
		corpus : Corpus
			Corpus to use.
		category : str
			Name of category column as it appears in original data frame.
		category_name : str
			Name of category to use.  E.g., "5-star reviews."
		not_category_name : str
			Name of everything that isn't in category.  E.g., "Below 5-star reviews".
		target_term : str
			Word or phrase for semantic similarity comparison
		word2vec : word2vec.Word2Vec
		  Gensim-compatible Word2Vec model of lower-cased corpus. If none, o
		  ne will be trained using Word2VecFromParsedCorpus(corpus).train()
		alpha : float, default = 0.01
			Uniform dirichlet prior for p-value calculation
		max_p_val : float, default = 0.05
			Max p-val to use find set of terms for similarity calculation

		Remaining arguments are from `produce_scattertext_explorer`.
		Returns
		-------
			str, html of visualization
		'''

	if word2vec is None:
		word2vec = Word2VecFromParsedCorpus(corpus).train()

	scores = []

	for tok in corpus._term_idx_store._i2val:
		try:
			scores.append(word2vec.similarity(target_term, tok.replace(' ', '_')))
		except:
			try:
				scores.append(np.mean([word2vec.similarity(target_term, tok_part)
				                       for tok_part in tok.split()]))
			except:
				scores.append(0)
	scores = np.array(scores)

	return produce_scattertext_explorer(corpus,
	                                    category,
	                                    category_name,
	                                    not_category_name,
	                                    scores=scores,
	                                    sort_by_dist=False,
	                                    reverse_sort_scores_for_not_category=False,
	                                    word_vec_use_p_vals=True,
	                                    term_significance=LogOddsRatioUninformativeDirichletPrior(alpha),
	                                    max_p_val=max_p_val,
	                                    p_value_colors=True,
	                                    **kwargs)


def word_similarity_explorer(corpus,
                             category,
                             category_name,
                             not_category_name,
                             target_term,
                             nlp=None,
                             alpha=0.01,
                             max_p_val=0.05,
                             **kwargs):
	'''
	Parameters
	----------
	corpus : Corpus
		Corpus to use.
	category : str
		Name of category column as it appears in original data frame.
	category_name : str
		Name of category to use.  E.g., "5-star reviews."
	not_category_name : str
		Name of everything that isn't in category.  E.g., "Below 5-star reviews".
	target_term : str
		Word or phrase for semantic similarity comparison
	nlp : spaCy-like parsing function
		E.g., spacy.en.English, whitespace_nlp, etc...
	alpha : float, default = 0.01
		Uniform dirichlet prior for p-value calculation
	max_p_val : float, default = 0.05
		Max p-val to use find set of terms for similarity calculation
	Remaining arguments are from `produce_scattertext_explorer`.
	Returns
	-------
		str, html of visualization
	'''

	if nlp is None:
		import spacy
		nlp = spacy.en.English()

	base_term = nlp(target_term)
	scores = np.array([base_term.similarity(nlp(tok))
	                   for tok
	                   in corpus._term_idx_store._i2val])
	return produce_scattertext_explorer(corpus,
	                                    category,
	                                    category_name,
	                                    not_category_name,
	                                    scores=scores,
	                                    sort_by_dist=False,
	                                    reverse_sort_scores_for_not_category=False,
	                                    word_vec_use_p_vals=True,
	                                    term_significance=LogOddsRatioUninformativeDirichletPrior(alpha),
	                                    max_p_val=max_p_val,
	                                    p_value_colors=True,
	                                    **kwargs)


def sparse_explorer(corpus,
                    category,
                    category_name,
                    not_category_name,
                    scores,
                    **kwargs):
	'''
	Parameters
	----------
	corpus : Corpus
		Corpus to use.
	category : str
		Name of category column as it appears in original data frame.
	category_name : str
		Name of category to use.  E.g., "5-star reviews."
	not_category_name : str
		Name of everything that isn't in category.  E.g., "Below 5-star reviews".
	scores : np.array
		Scores to display in visualization.  Zero scores are grey.

	Remaining arguments are from `produce_scattertext_explorer`.

	Returns
	-------
		str, html of visualization
	'''

	return produce_scattertext_explorer(
		corpus,
		category,
		category_name,
		not_category_name,
		scores=scores,
		sort_by_dist=False,
		grey_zero_scores=True,
		**kwargs)



import pandas as pd
from urllib.request import urlopen
import scattertext as st

def main():
	shisei = _parse_geutenberg('http://www.gutenberg.org/files/31617/31617-0.txt')
	horadanshaku = _parse_geutenberg('http://www.gutenberg.org/files/34084/34084-0.txt')
	df = pd.DataFrame({'text': [shisei, horadanshaku],
	                   'title': ['Shisei', 'Horadanshaku tabimiyage'],
	                   'author': ['Akutagawa Ryunosuke', 'Kuni Sasaki']})

	df['text'] = df['text'].apply(st.japanese_nlp)
	corpus = st.CorpusFromParsedDocuments(df,
	                                      category_col='title',
	                                      parsed_col='text').build()
	html = st.produce_scattertext_explorer(corpus,
	                                       category='Shisei',
	                                       category_name='Shisei',
	                                       not_category_name='Horadanshaku tabimiyage',
	                                       minimum_term_frequency=5,
	                                       width_in_pixels=1000,
	                                       metadata=df['title'] + ' by ' + df['author'],
	                                       asian_mode=True)
	open('./demo_japanese.html', 'w').write(html)
	print('Open ./demo_japanese.html in Chrome or Firefox.')


def _parse_geutenberg(url):
	return (urlopen(url)
	        .read()
	        .decode('utf-8')
	        .split("Transcriber's Notes")[0]
	        .split('-------------------------------------------------------')[-1])


if __name__ == '__main__':
	main()



import spacy
from sklearn.linear_model import Lasso
from sklearn.linear_model import LogisticRegression

from scattertext import SampleCorpora, sparse_explorer
from scattertext.CorpusFromPandas import CorpusFromPandas

nlp = spacy.en.English()
convention_df = SampleCorpora.ConventionData2012.get_data()
corpus = CorpusFromPandas(convention_df,
                          category_col='party',
                          text_col='text',
                          nlp=nlp).build()
scores = corpus.get_logreg_coefs('democrat',
                                 LogisticRegression(penalty='l1', C=10, max_iter=10000, n_jobs=-1))
html = sparse_explorer(corpus,
                       category='democrat',
                       category_name='Democratic',
                       not_category_name='Republican',
                       scores = scores,
                       minimum_term_frequency=5,
                       width_in_pixels=1000,
                       metadata=convention_df['speaker'])
open('./demo_sparse.html', 'wb').write(html.encode('utf-8'))
print('Open ./demo_sparse.html in Chrome or Firefox.')



import itertools

from gensim.models import word2vec, Phrases

from scattertext.ParsedCorpus import ParsedCorpus


class Word2VecFromParsedCorpus(object):
	def __init__(self, corpus, word2vec_model=None):
		assert isinstance(corpus, ParsedCorpus)
		assert word2vec_model is None or isinstance(word2vec_model, word2vec.Word2Vec)
		self.corpus = corpus
		self.model = self._get_word2vec_model(word2vec_model)

	def _get_word2vec_model(self, word2vec_model):
		return (self._default_word2vec_model()
		        if word2vec_model is None
		        else word2vec_model)

	def _default_word2vec_model(self):
		return word2vec.Word2Vec(size=100, alpha=0.025, window=5, min_count=5,
		                         max_vocab_size=None, sample=0, seed=1, workers=1, min_alpha=0.0001,
		                         sg=1, hs=1, negative=0, cbow_mean=0, iter=1, null_word=0,
		                         trim_rule=None, sorted_vocab=1)

	def train(self, epochs=2000):
		'''
		Parameters
		----------
		epochs int, number of epochs to train for.  Default is 2000.

		Returns
		-------
		A trained word2vec model.
		'''

		self._scan_and_build_vocab()
		self.model.train(self._get_line_sentences_for_word2vec(),
		                 total_examples=self.model.corpus_count,
		                 epochs=epochs)
		return self.model

	def _scan_and_build_vocab(self):
		self.model.scan_vocab(self._get_line_sentences_for_word2vec())
		self.model.build_vocab(self._get_line_sentences_for_word2vec())

	def _get_line_sentences_for_word2vec(self):
		return itertools.chain(*[[[t.lower_ for t in sent if not t.is_punct]
		                          for sent in doc.sents]
		                         for doc in self.corpus.get_parsed_docs()])


class Word2VecFromParsedCorpusBigrams(Word2VecFromParsedCorpus):
	def _scan_and_build_vocab(self):
		bigram_transformer = Phrases(self._get_line_sentences_for_word2vec())
		self.model.scan_vocab(bigram_transformer[self._get_line_sentences_for_word2vec()])
		self.model.build_vocab(bigram_transformer[self._get_line_sentences_for_word2vec()])



import numpy as np
import spacy
from sklearn.linear_model import LogisticRegression

from scattertext import SampleCorpora
from scattertext import produce_scattertext_explorer
from scattertext.CorpusFromPandas import CorpusFromPandas

nlp = spacy.en.English()
convention_df = SampleCorpora.ConventionData2012.get_data()
corpus = CorpusFromPandas(convention_df,
                          category_col='party',
                          text_col='text',
                          nlp=nlp).build()

term_freq_df = corpus.get_term_freq_df()

def scale(ar):
	return (ar - ar.min()) / (ar.max() - ar.min())

def zero_centered_scale(ar):
	ar[ar > 0] = scale(ar[ar > 0])
	ar[ar < 0] = -scale(-ar[ar < 0])
	return (ar + 1) / 2.


frequencies_scaled = scale(np.log(term_freq_df.sum(axis=1).values))
scores = corpus.get_logreg_coefs('democrat',
                                 LogisticRegression(penalty='l2', C=10, max_iter=10000, n_jobs=-1))
scores_scaled = zero_centered_scale(scores)

html = produce_scattertext_explorer(corpus,
                                    category='democrat',
                                    category_name='Democratic',
                                    not_category_name='Republican',
                                    minimum_term_frequency=5,
                                    pmi_threshold_coefficient=4,
                                    width_in_pixels=1000,
                                    x_coords=frequencies_scaled,
                                    y_coords=scores_scaled,
                                    scores=scores,
                                    sort_by_dist=False,
                                    metadata=convention_df['speaker'],
                                    x_label='Log frequency',
                                    y_label='L2-penalized logistic regression coef')
fn = 'demo_custom_coordinates.html'
open(fn, 'wb').write(html.encode('utf-8'))
print('Open %s in Chrome or Firefox.' % fn)



import numpy as np
import pandas as pd
from scipy.stats import rankdata


def scale(vec, terms=None, other_vec=None):
	return (vec - vec.min()) / (vec.max() - vec.min())


def scale_standardize(vec, terms=None, other_vec=None):
	to_ret = (vec - vec.mean()) / vec.std()
	to_ret += to_ret.min() + 1
	# to_ret = np.log(to_ret)/np.log(2)
	to_ret += to_ret.min()
	return to_ret


def log_scale_standardize(vec, terms=None, other_vec=None):
	vec_ss = (np.log(vec + 1) / np.log(2))
	vec_ss = (vec_ss - vec_ss.mean()) / vec_ss.std()
	return _scale_0_to_1(vec_ss)


def sqrt_scale_standardize(vec, terms=None, other_vec=None):
	vec_ss = np.sqrt(vec)
	vec_ss = (vec_ss - vec_ss.mean()) / vec_ss.std()
	return _scale_0_to_1(vec_ss)

def power_scale_standardize_factory(alpha):
	def f(vec, terms=None, other_vec=None):
		vec_ss = np.power(vec, 1./alpha)
		vec_ss = (vec_ss - vec_ss.mean()) / vec_ss.std()
		return _scale_0_to_1(vec_ss)
	return f

'''
from statsmodels.distributions import ECDF
def ecdf_scale_standardize(vec):
	vec_ss = ECDF(vec)(vec)
	vec_ss = (vec_ss - vec_ss.min()) / (vec_ss.max() - vec_ss.min())
	return vec_ss
'''


def power_scale_factory(alpha):
	def f(vec, terms=None, other_vec=None):
		return _scale_0_to_1(np.power(vec, 1./alpha))
	return f

def sqrt_scale(vec, terms=None, other_vec=None):
	return _scale_0_to_1(np.sqrt(vec))

def log_scale(vec, terms=None, other_vec=None):
	return _scale_0_to_1(np.log(vec))

def percentile(vec, terms=None, other_vec=None):
	vec_ss = rankdata(vec, method='average') * (1. / len(vec))
	return _scale_0_to_1(vec_ss)


def percentile_ordinal(vec, terms=None, other_vec=None):
	vec_ss = rankdata(vec, method='ordinal') * (1. / len(vec))
	return _scale_0_to_1(vec_ss)


def percentile_min(vec, terms=None, other_vec=None):
	vec_ss = rankdata(vec, method='min') * (1. / len(vec))
	return _scale_0_to_1(vec_ss)


def percentile_alphabetical(vec, terms, other_vec=None):
	scale_df = pd.DataFrame({'scores': vec, 'terms': terms})
	if other_vec is not None:
		scale_df['others'] = other_vec
	else:
		scale_df['others'] = 0
	vec_ss = (scale_df
	          .sort_values(by=['scores', 'terms', 'others'],
	                       ascending=[True, True, False])
	          .reset_index()
	          .sort_values(by='index')
	          .index)
	return _scale_0_to_1(vec_ss)


def _scale_0_to_1(vec_ss):
	vec_ss = (vec_ss - vec_ss.min()) * 1. / (vec_ss.max() - vec_ss.min())
	return vec_ss



import re

from scattertext.emojis.ProcessedEmojiStructure import VALID_EMOJIS

'''
This is a fast but awful partial implementation of Spacy.  It's useful for testing.
'''


class Tok:
	def __init__(self, pos, lem, low, ent_type, tag, is_punct=False):
		self.pos_ = pos
		self.lemma_ = lem
		self.lower_ = low
		self.ent_type_ = ent_type
		self.tag_ = tag
		self.is_punct = is_punct

	def __str__(self):
		return self.lower_

	def __len__(self):
		return len(self.lower_)


class Doc:
	def __init__(self, sents, raw=None, noun_chunks = []):
		self.sents = sents
		if raw is None:
			self.string = ' '.join(
				''.join([tok.lower_ for tok in sent]) for sent in sents
			)
		else:
			self.string = raw
		self.text = self.string
		self.noun_chunks = noun_chunks

	def __str__(self):
		return self.string

	def __repr__(self):
		return self.string

	def __iter__(self):
		for sent in self.sents:
			for tok in sent:
				yield tok


def whitespace_nlp(doc, entity_type=None, tag_type=None):
	toks = _regex_parse_sentence(doc, entity_type, tag_type)
	return Doc([toks])




def _regex_parse_sentence(doc, entity_type, tag_type):
	toks = _toks_from_sentence(doc, entity_type, tag_type)
	return toks


def _toks_from_sentence(doc, entity_type, tag_type):
	toks = []
	for tok in re.split(r"(\W)", doc):
		pos = 'WORD'
		if tok.strip() == '':
			pos = 'SPACE'
		elif re.match('^\W+$', tok):
			pos = 'PUNCT'
		toks.append(Tok(pos,
		                tok[:2].lower(),
		                tok.lower(),
		                ent_type='' if entity_type is None else entity_type.get(tok, ''),
		                tag='' if tag_type is None else tag_type.get(tok, '')))
	return toks


def tweet_tokenzier_factory(tweet_tokenizer):
	'''
	Parameters
	----------
	tweet_tokenizer : nltk.tokenize.TweetTokenizer instance

	Returns
	-------
	Doc of tweets

	Notes
	-------
	Requires NLTK to be installed :(
	'''
	def tokenize(text):
		toks = []
		for tok in tweet_tokenizer.tokenize(text):
			pos = 'WORD'
			if tok.strip() == '':
				pos = 'SPACE'
			elif ord(tok[0]) in VALID_EMOJIS:
				pos = 'EMJOI'
			elif re.match('^\W+$', tok):
				pos = 'PUNCT'
			toks.append(Tok(pos,
			                tok.lower(),
			                tok.lower(),
			                ent_type='',
			                tag='' ))
		return Doc([toks], text)
	return tokenize


def whitespace_nlp_with_sentences(doc, entity_type=None, tag_type=None):
	pat = re.compile(r'([A-Z][^\.!?]*[\.!?])', re.M)
	sents = []
	for sent in pat.findall(doc):
		toks = []
		for tok in re.split(r"(\W)", sent):
			pos = 'WORD'
			if tok.strip() == '':
				pos = 'SPACE'
			elif re.match('^\W+$', tok):
				pos = 'PUNCT'
			toks.append(Tok(pos,
			                tok[:2].lower(),
			                tok.lower(),
			                ent_type='' if entity_type is None else entity_type.get(tok, ''),
			                tag='' if tag_type is None else tag_type.get(tok, '')))
		sents.append(toks)
		toks = []
	return Doc(sents, doc)



import numpy as np
from scipy.stats import norm, rankdata

from scattertext.Common import DEFAULT_SCALER_ALGO, DEFAULT_BETA


class InvalidScalerException(Exception):
	pass


class ScoreBalancer(object):
	@staticmethod
	def balance_scores(cat_scores, not_cat_scores):
		scores = ScoreBalancer.balance_scores_and_dont_scale(cat_scores, not_cat_scores)
		return ScoreBalancer._zero_centered_scale(scores)

	@staticmethod
	def balance_scores_and_dont_scale(cat_scores, not_cat_scores):
		'''
		median = np.median(cat_scores)
		scores = np.zeros(len(cat_scores)).astype(np.float)
		scores[cat_scores > median] = cat_scores[cat_scores > median]
		not_cat_mask = cat_scores < median if median != 0 else cat_scores <= median
		scores[not_cat_mask] = -not_cat_scores[not_cat_mask]
		'''
		scores = np.zeros(len(cat_scores)).astype(np.float)
		scores[cat_scores > not_cat_scores] = cat_scores[cat_scores > not_cat_scores]
		scores[cat_scores < not_cat_scores] = -not_cat_scores[cat_scores < not_cat_scores]
		return scores

	@staticmethod
	def _zero_centered_scale(ar):
		ar[ar > 0] = ScoreBalancer._scale(ar[ar > 0])
		ar[ar < 0] = -ScoreBalancer._scale(-ar[ar < 0])
		return (ar + 1) / 2.

	@staticmethod
	def _scale(ar):
		if len(ar) == 0:
			return ar
		if ar.min() == ar.max():
			return np.full(len(ar), 0.5)
		return (ar - ar.min()) / (ar.max() - ar.min())


class ScaledFScore(object):
	@staticmethod
	def get_scores(cat_word_counts, not_cat_word_counts,
	               scaler_algo=DEFAULT_SCALER_ALGO, beta=DEFAULT_BETA):
		''' Computes scaled-fscores
		Parameters
		----------
		category : str
			category name to score
		scaler_algo : str
			Function that scales an array to a range \in [0 and 1]. Use 'percentile', 'normcdf'. Default.
		beta : float
			Beta in (1+B^2) * (Scale(P(w|c)) * Scale(P(c|w)))/(B^2*Scale(P(w|c)) + Scale(P(c|w))). Default.
		Returns
		-------
			np.array of harmonic means of scaled P(word|category)
			 and scaled P(category|word) for >median half of scores.  Low scores are harmonic means
			 of scaled P(word|~category) and scaled P(~category|word).  Array is squashed to between
			 0 and 1, with 0.5 indicating a median score.
		'''

		cat_scores = ScaledFScore.get_scores_for_category(cat_word_counts, not_cat_word_counts, scaler_algo, beta)
		not_cat_scores = ScaledFScore.get_scores_for_category(not_cat_word_counts, cat_word_counts,
		                                                      scaler_algo, beta)
		return ScoreBalancer.balance_scores(cat_scores, not_cat_scores)

	@staticmethod
	def get_scores_for_category(cat_word_counts, not_cat_word_counts,
	                            scaler_algo=DEFAULT_SCALER_ALGO, beta=DEFAULT_BETA):
		''' Computes scaled-fscores
		Parameters
		----------
		category : str
			category name to score
		scaler_algo : str
			Function that scales an array to a range \in [0 and 1]. Use 'percentile', 'normcdf'. Default normcdf
		beta : float
			Beta in (1+B^2) * (Scale(P(w|c)) * Scale(P(c|w)))/(B^2*Scale(P(w|c)) + Scale(P(c|w))). Defaults to 1.
		Returns
		-------
			np.array of harmonic means of scaled P(word|category) and scaled P(category|word).
		'''
		assert beta > 0
		precision = (cat_word_counts * 1. / (cat_word_counts + not_cat_word_counts))
		recall = cat_word_counts * 1. / cat_word_counts.sum()
		precision_normcdf = ScaledFScore._safe_scaler(scaler_algo, precision)
		recall_normcdf = ScaledFScore._safe_scaler(scaler_algo, recall)
		scores = (1 + beta ** 2) * (precision_normcdf * recall_normcdf) \
		       / ((beta ** 2) * precision_normcdf + recall_normcdf)
		scores[np.isnan(scores)] = 0.
		return scores

	@staticmethod
	def _get_scaled_f_score_from_counts(cat_word_counts, not_cat_word_counts, scaler_algo, beta=DEFAULT_BETA):
		p_word_given_category = cat_word_counts.astype(np.float) / cat_word_counts.sum()
		p_category_given_word = cat_word_counts * 1. / (cat_word_counts + not_cat_word_counts)
		scores \
			= ScaledFScore._get_harmonic_mean_of_probabilities_over_non_zero_in_category_count_terms \
			(cat_word_counts, p_category_given_word, p_word_given_category, scaler_algo, beta)
		return scores

	@staticmethod
	def _safe_scaler(algo, ar):
		if algo == 'none':
			return ar
		scaled_ar = ScaledFScore._get_scaler_function(algo)(ar)
		if np.isnan(scaled_ar).any():
			return ScaledFScore._get_scaler_function('percentile')(scaled_ar)
		return scaled_ar

	@staticmethod
	def _get_scaler_function(scaler_algo):
		scaler = None
		if scaler_algo == 'normcdf':
			scaler = lambda x: norm.cdf(x, x.mean(), x.std())
		elif scaler_algo == 'percentile':
			scaler = lambda x: rankdata(x).astype(np.float64) / len(x)
		elif scaler_algo == 'none':
			scaler = lambda x: x
		else:
			raise InvalidScalerException("Invalid scaler alogrithm.  Must be either percentile or normcdf.")
		return scaler



import io
from zipfile import ZipFile

import agefromname
import nltk
import pandas as pd
import urllib.request

import scattertext as st

nlp = st.tweet_tokenzier_factory(nltk.tokenize.TweetTokenizer())
with ZipFile(io.BytesIO(urllib.request.urlopen(
		'http://followthehashtag.com/content/uploads/USA-Geolocated-tweets-free-dataset-Followthehashtag.zip'
).read())) as zf:
	df = pd.read_excel(zf.open('dashboard_x_usa_x_filter_nativeretweets.xlsx'))
df['parse'] = df['Tweet content'].apply(nlp)

male_prob = agefromname.AgeFromName().get_all_name_male_prob()

df['first_name'] = df['User Name'].apply(lambda x: x.split()[0].lower() if type(x) == str and len(x.split()) > 0 else x)
df_aug = pd.merge(df, male_prob, left_on='first_name', right_index=True)
df_aug['gender'] = df_aug['prob'].apply(lambda x: 'm' if x > 0.9 else 'f' if x < 0.1 else '?')
df_mf = df_aug[df_aug['gender'].isin(['m', 'f'])]

corpus = st.CorpusFromParsedDocuments(df_mf,
                                      parsed_col='parse',
                                      category_col='gender',
                                      feats_from_spacy_doc=st.FeatsFromSpacyDocOnlyEmoji()
                                      ).build()

html = st.produce_scattertext_explorer(corpus,
                                       category='f',
                                       category_name='Female',
                                       not_category_name='Male',
                                       use_full_doc=True,
                                       metadata=df_mf['User Name'] + ' (@' + df_mf['Nickname'] + ') ' + df_mf[
	                                       'Date'].astype(str),
                                       show_characteristic=False,
                                       width_in_pixels=1000)
file_name = "EmojiGender.html"
open(file_name, 'wb').write(html.encode('utf-8'))



import json
import pkgutil

from scattertext.Common import DEFAULT_D3_URL, DEFAULT_D3_SCALE_CHROMATIC


class InvalidProtocolException(Exception):
	pass


class HTMLVisualizationAssembly:
	def __init__(self,
	             visualization_data,
	             width_in_pixels=None,
	             height_in_pixels=None,
	             max_snippets=None,
	             color=None,
	             grey_zero_scores=False,
	             sort_by_dist=True,
	             reverse_sort_scores_for_not_category=True,
	             use_full_doc=False,
	             asian_mode=False,
	             use_non_text_features=False,
	             show_characteristic=True,
	             word_vec_use_p_vals=False,
	             max_p_val=0.05,
	             save_svg_button=False,
	             p_value_colors=False,
	             x_label=None,
	             y_label=None):
		'''

		Parameters
		----------
		visualization_data : dict
			From ScatterChart or a descendant
		width_in_pixels : int, optional
			width of viz in pixels, if None, default to JS's choice
		height_in_pixels : int, optional
			height of viz in pixels, if None, default to JS's choice
		max_snippets : int, optional
			max snippets to snow when temr is clicked, Defaults to show all
		color : str, optional
		  d3 color scheme
		grey_zero_scores : bool, optional
		  If True, color points with zero-scores a light shade of grey.  False by default.
		sort_by_dist : bool, optional
			sort by distance or score, default True
		reverse_sort_scores_for_not_category : bool, optional
			If using a custom score, score the not-category class by
			lowest-score-as-most-predictive. Turn this off for word vectory
			or topic similarity. Default True.
		use_full_doc : bool, optional
			use full document instead of sentence in snippeting
		use_non_text_features : bool, default False
			Show non-bag-of-words features (e.g., Empath) instaed of text.  False by default.
		show_characteristic: bool, default True
			Show characteristic terms on the far left-hand side of the visualization
		word_vec_use_p_vals: bool, default False
			Use category-associated p-values to determine which terms to give word
			vec scores.
		max_p_val : float, default = 0.05
			Max p-val to use find set of terms for similarity calculation, if
			word_vec_use_p_vals is True.
		save_svg_button : bool, default False
			Add a save as SVG button to the page.
		p_value_colors : bool, default False
		  Color points differently if p val is above 1-max_p_val, below max_p_val, or
		   in between.
		x_label : str, default None
			If present, use as the x-axis label
		y_label : str, default None
			If present, use as the y-axis label
		'''
		self._visualization_data = visualization_data
		self._width_in_pixels = width_in_pixels
		self._height_in_pixels = height_in_pixels
		self._max_snippets = max_snippets
		self._color = color
		self._sort_by_dist = sort_by_dist
		self._use_full_doc = use_full_doc
		self._asian_mode = asian_mode
		self._grey_zero_scores = grey_zero_scores
		self._use_non_text_features = use_non_text_features
		self._show_characteristic = show_characteristic
		self._word_vec_use_p_vals = word_vec_use_p_vals
		self._max_p_val = max_p_val
		self._save_svg_button = save_svg_button
		self._reverse_sort_scores_for_not_category = reverse_sort_scores_for_not_category
		self._p_value_colors = p_value_colors
		self._x_label = x_label
		self._y_label = y_label

	def to_html(self,
	            protocol='http',
	            d3_url = None,
	            d3_scale_chromatic_url = None):
		'''
		Parameters
		----------
		protocol, 'http' or 'https' for including external urls
		d3_url, str, None by default.  The url (or path) of d3.
			URL of d3, to be inserted into <script src="..."/>
		  By default, this is `DEFAULT_D3_URL` declared in `HTMLVisualizationAssembly`.
		d3_scale_chromatic_url, str, None by default.
		  URL of d3_scale_chromatic_url, to be inserted into <script src="..."/>
		  By default, this is `DEFAULT_D3_SCALE_CHROMATIC` declared in `HTMLVisualizationAssembly`.

		Returns
		-------
		str, the html file representation

		'''
		self._ensure_valid_protocol(protocol)
		javascript_to_insert = '\n'.join([
			self._full_content_of_javascript_files(),
			self._visualization_data.to_javascript(),
			self._call_build_visualization_in_javascript()
		])

		if d3_url is None:
			d3_url = DEFAULT_D3_URL

		if d3_scale_chromatic_url is None:
			d3_scale_chromatic_url = DEFAULT_D3_SCALE_CHROMATIC


		html_file = (self._full_content_of_html_file()
		             .replace('<!-- INSERT SCRIPT -->', javascript_to_insert, 1)
		             .replace('<!--D3URL-->', d3_url, 1)
		             .replace('<!--D3SCALECHROMATIC-->', d3_scale_chromatic_url)
		             # .replace('<!-- INSERT D3 -->', self._get_packaged_file_content('d3.min.js'), 1)
		             )

		extra_libs = ''
		if self._save_svg_button:
			# extra_libs = '<script src="https://cdn.rawgit.com/edeno/d3-save-svg/gh-pages/assets/d3-save-svg.min.js" charset="utf-8"></script>'
			extra_libs = ''
		html_file = (html_file
		             .replace('<!-- EXTRA LIBS -->', extra_libs, 1)
		             .replace('http://', protocol + '://'))
		return html_file

	def _ensure_valid_protocol(self, protocol):
		if protocol not in ('https', 'http'):
			raise InvalidProtocolException(
				"Invalid protocol: %s.  Protocol must be either http or https." % (protocol))

	def _full_content_of_html_file(self):
		return pkgutil.get_data('scattertext',
		                        'data/viz/scattertext.html').decode('utf-8')

	def _full_content_of_javascript_files(self):
		return '; \n \n '.join([self._get_packaged_file_content(script_name)
		                        for script_name in [
			                        #'d3.min.js',
			                        #'d3-scale-chromatic.v1.min.js',
			                        'rectangle-holder.js',  # 'range-tree.js',
			                        'main.js',
		                        ]])

	def _get_packaged_file_content(self, file_name):
		return pkgutil.get_data('scattertext',
		                        'data/viz/scripts/' + file_name).decode('utf-8')

	def _call_build_visualization_in_javascript(self):
		def js_default_value(x):
			return 'undefined' if x is None else str(x)

		def js_default_string(x):
			return 'undefined' if x is None else json.dumps(str(x))

		def js_default_value_to_null(x):
			return 'null' if x is None else str(x)

		def js_bool(x):
			return 'true' if x else 'false'

		def js_float(x):
			return str(float(x))

		arguments = [js_default_value(self._width_in_pixels),
		             js_default_value(self._height_in_pixels),
		             js_default_value_to_null(self._max_snippets),
		             js_default_value_to_null(self._color),
		             js_bool(self._sort_by_dist),
		             js_bool(self._use_full_doc),
		             js_bool(self._grey_zero_scores),
		             js_bool(self._asian_mode),
		             js_bool(self._use_non_text_features),
		             js_bool(self._show_characteristic),
		             js_bool(self._word_vec_use_p_vals),
		             js_bool(self._save_svg_button),
		             js_bool(self._reverse_sort_scores_for_not_category),
		             js_float(self._max_p_val),
		             js_bool(self._p_value_colors),
		             js_default_string(self._x_label),
		             js_default_string(self._y_label)]
		return 'buildViz(' + ','.join(arguments) + ');'



import spacy

from scattertext import SampleCorpora, produce_scattertext_explorer
from scattertext.CorpusFromPandas import CorpusFromPandas
from scattertext.termscoring.LogOddsUniformativePriorScore import LogOddsUninformativePriorScore

nlp = spacy.en.English()
convention_df = SampleCorpora.ConventionData2012.get_data()
corpus = CorpusFromPandas(convention_df,
                          category_col='party',
                          text_col='text',
                          nlp=nlp).build()
term_freq_df = corpus.get_term_freq_df()
scores = -(LogOddsUninformativePriorScore
           .get_thresholded_score(term_freq_df['democrat freq'],
                                  term_freq_df['republican freq'],
                                  alpha_w=2.,
                                  threshold=0.05))
html = produce_scattertext_explorer(corpus,
                                    category='democrat',
                                    category_name='Democratic',
                                    not_category_name='Republican',
                                    scores=scores,
                                    sort_by_dist=False,
                                    grey_zero_scores=True,
                                    minimum_term_frequency=5,
                                    width_in_pixels=1000,
                                    metadata=convention_df['speaker'])
open('./demo_insignificant_greyed_out.html', 'wb').write(html.encode('utf-8'))
print('Open ./demo_insignificant_greyed_out.html in Chrome or Firefox.')



import spacy

from scattertext import SampleCorpora, word_similarity_explorer
from scattertext.CorpusFromPandas import CorpusFromPandas


def main():
	nlp = spacy.en.English()
	convention_df = SampleCorpora.ConventionData2012.get_data()
	corpus = CorpusFromPandas(convention_df,
	                          category_col='party',
	                          text_col='text',
	                          nlp=nlp).build()
	html = word_similarity_explorer(corpus,
	                                category='democrat',
	                                category_name='Democratic',
	                                not_category_name='Republican',
	                                target_term='jobs',
	                                minimum_term_frequency=5,
	                                width_in_pixels=1000,
	                                metadata=convention_df['speaker'],
	                                alpha=0.01,
	                                max_p_val=0.05,
	                                save_svg_button=True)
	open('./demo_similarity.html', 'wb').write(html.encode('utf-8'))
	print('Open ./demo_similarlity.html in Chrome or Firefox.')


if __name__ == '__main__':
	main()



# -*- coding: utf-8 -*-
import numpy as np
from scipy.stats import norm

from scattertext.termsignificance.TermSignificance import TermSignificance


class LogOddsRatioUninformativeDirichletPrior(TermSignificance):
	def __init__(self, alpha_w=0.01):
		'''
		Parameters
		----------
		alpha_w : np.float
			The constant prior.
		'''
		self.alpha_w = alpha_w



	def get_p_vals(self, X):
		'''
		Implements the log-odds-ratio with an uninformative dirichlet prior from
		Monroe, B. L., Colaresi, M. P., & Quinn, K. M. (2008). Fightin' words: Lexical feature selection and evaluation for identifying the content of political conflict. Political Analysis, 16(4), 372–403.

		Parameters
		----------
		X : np.array
			Array of word counts, shape (N, 2) where N is the vocab size.  X[:,0] is the
			positive class, while X[:,1] is the negative class. None by default

		Returns
		-------
		np.array of p-values

		'''

		# Equation 16 of Monroe et al. 2008.
		lod_odds_ratio = self.get_log_odds_with_prior(X)

		# Equation 19
		a_w = self.alpha_w
		n_pos, n_neg = X.sum(axis=0)
		a_0 = X.shape[0] * a_w
		std_lod_odds_ratio = (1. / (X[:, 0] + a_w)
		                      + 1. / (n_pos + a_0 - X[:, 0] - a_w)
		                      + 1. / (X[:, 1] + a_w)
		                      + 1. / (n_neg + a_0 - X[:, 1] - a_w))

		# Equation 22
		z_scores = lod_odds_ratio / np.sqrt(std_lod_odds_ratio)

		return norm.cdf(z_scores)

	def get_log_odds_with_prior(self, X):
		a_w = self.alpha_w
		n_pos, n_neg = X.sum(axis=0)
		a_0 = X.shape[0] * a_w
		return (np.log((X[:, 0] + a_w) / (n_pos + a_0 - X[:, 0] - a_w))
		        - np.log((X[:, 1] + a_w) / (n_neg + a_0 - X[:, 1] - a_w)))





from unittest import TestCase

import numpy as np

from scattertext.termscoring.ScaledFScore import ScaledFScore


class TestScaledFScore(TestCase):
	def test_get_scores(self):
		cat_counts, not_cat_counts = self._get_counts()
		scores = ScaledFScore.get_scores(cat_counts, not_cat_counts)
		np.testing.assert_almost_equal(scores,
		                               np.array([0.2689108, 0., 0.2689108, 0.1266617, 1.,
		                                         0.5, 0.5590517, 0.5, 0.5, 0.5720015]))

	def test_get_scores_zero_all_same(self):
		cat_counts = np.array([0, 0, 0, 0, 0, 0, 1, 2])
		not_cat_counts = np.array([1, 1, 2, 1, 1, 1, 1, 2])
		scores = ScaledFScore.get_scores(cat_counts, not_cat_counts)
		np.testing.assert_almost_equal(scores, [0.5, 0.5, 0, 0.5, 0.5, 0.5, 0.5, 1.])

	def test_get_scores_zero_median(self):
		cat_counts = np.array([0, 0, 0, 0, 0, 0, 1, 2])
		not_cat_counts = np.array([1, 1, 2, 1, 1, 1, 1, 3])
		ScaledFScore.get_scores(cat_counts, not_cat_counts)

	def get_scores_for_category(self):
		cat_counts, not_cat_counts = self._get_counts()
		scores = ScaledFScore.get_scores_for_category(cat_counts, not_cat_counts)
		np.testing.assert_almost_equal(scores,
		                               [0.23991183969723384, 0.24969810634506373, 0.23991183969723384,
		                                0.27646711056272855, 0.92885244834997516, 0.42010144843632563,
		                                0.49166017105966719, 0.0, 0.0, 0.50262304057984664])

	def _get_counts(self):
		cat_counts = np.array([1, 5, 1, 9, 100, 1, 1, 0, 0, 2])
		not_cat_counts = np.array([100, 510, 100, 199, 0, 1, 0, 1, 1, 0])

		return cat_counts, not_cat_counts
